package com.xylink.util;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.RandomStringUtils;

import javax.crypto.Cipher;
import java.nio.charset.StandardCharsets;
import java.security.Key;
import java.security.KeyFactory;
import java.security.spec.X509EncodedKeySpec;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Base64;
import java.util.regex.Pattern;

public class AuthcodeUtil {


    public static final String SEPARATOR = ",";
    // private static DateTimeFormatter timeStrFormatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmssSSS");
    public static DateTimeFormatter timeStrFormatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
    public static Pattern extUserIdMatcher = Pattern.compile("^[0-9a-zA-Z_-]{0,50}$");
    public static final String KEY_ALGORITHM = "RSA";

    /**
     * RSA加密方式获取authcode
     * @param externalUserId
     * @param pubKey
     * @return authCode usable in nowtime +/- 2 minutes （避免云机器时间差过大）
     * @throws Exception
     */
    public static String generateAuthCodeByRSA(String externalUserId, String pubKey) {
        if (!matchExtUserId(externalUserId)){
            System.out.println(externalUserId + " externalUserId formatter error");
            return null;
        }
        String ramdom = RandomStringUtils.random(11, true, false);
        LocalDateTime dateTime = LocalDateTime.now();
        String timeStr = dateTime.format(timeStrFormatter);
        String content = externalUserId + SEPARATOR + timeStr+ SEPARATOR+ramdom;
        System.out.println("加密前："+content);
        return encryptByPubKey4String(content, pubKey);
    }

    public static byte[] rsaEncryptByPublicKey(byte[] data, String publicKey)
            throws Exception {
        //得到公钥
        byte[] keyBytes = Base64.getDecoder().decode(publicKey);
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        Key publicK = keyFactory.generatePublic(x509KeySpec);
        // 对数据加密
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.ENCRYPT_MODE, publicK);
        return cipher.doFinal(data);
    }

    public static byte[] encryptByPubKey4Byte(String data, String publicKey) {
        if (data == null || data.equals("")) {
            return null;
        }
        byte[] publicEncrypData = null;
        try {
            //1、data base64 encode
            String dataBase64Encoded = Base64.getEncoder().encodeToString(data.getBytes(StandardCharsets.UTF_8));
            System.out.println("rsaencrypt dataBase64Encoded "+dataBase64Encoded);
            //2、encrypt
            publicEncrypData = rsaEncryptByPublicKey(dataBase64Encoded.getBytes(StandardCharsets.UTF_8), publicKey);
        } catch (Exception e) {
            System.out.println("rsaEncryptByPubKey4Byte error "+e);
        }
        return publicEncrypData;
    }

    public static String encryptByPubKey4String(String data, String publicKey) {
        byte[] bytes = encryptByPubKey4Byte(data, publicKey);
        return Hex.encodeHexString(bytes);
    }

    public static boolean matchExtUserId(String extUserId){
        System.out.println(extUserId.length()+" extUserId ->"+extUserId);
        return extUserIdMatcher.matcher(extUserId).find();
    }


    public static void main(String[] args) throws Exception {
        String pubkeyStr = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCQ2jn89Cxqxom6eyUKxf6k6k1IiDcMXrn9FHbfoZ9f4W+6zGQNwCWnYdTM5RXfR/L2XjmZMm8Cac/mt//BGowhmVhAESdrOex05h2ijK6aiz4o4I3UPe7ghKsvBNUy8FSSQZ/Y69Xbg6+jkz1JEPkkGVZRikrOeYnLDTusbc+5XwIDAQAB";
        String data = "u"+System.currentTimeMillis();
        String authcode = generateAuthCodeByRSA(data,pubkeyStr);
        System.out.println("加密后: "+authcode);
    }

}
